/* $Id: lmin_main.c,v 3.3 1998/03/21 00:35:53 kas Exp $ */

/* Main function for local-minimizing program */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "config.h"
#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
#include "getopt.h"
#endif

#include "ff.h"
#include "pval.h"

static char *program_name;

static void usage();

int main(argc,argv)
int argc;
char **argv;
{
	REAL vdw14 = -1.0, eel14 = -1.0, diel_lin = -1.0, diel_const = -1.0;
	REAL cutoff = -1.0;
	REAL accuracy = 1.0e-5;
	int seed = 0;
	force_field *ffp;
	potential_value *pvp;
	char *outfile = NULL;
	char *infile = NULL;
#ifndef NONTRANSFORMED_ONLY
	char *ptmpl = NULL, *atmpl = NULL, *ttmpl = NULL;
#endif
	program_name = *argv;

	while(1)
	{
		int c, opt_ind;
		static struct option long_opts[] = {
			{ "help", 0, 0, 'h' },
			{ "version", 0, 0, 'v' },
			{ "vdw14", 1, 0, 'w' },
			{ "eel14", 1, 0, 'e' },
			{ "diel-lin", 1, 0, 'l' },
			{ "diel-const", 1, 0, 'c' },
			{ "cutoff", 1, 0, 'C' },
			{ "seed", 1, 0, 's' },
			{ "outfile", 1, 0, 'o' },
			{ "begin-coords", 1, 0, 'b' },
#ifndef NONTRANSFORMED_ONLY
			{ "pair-template", 1, 0, 'P' },
			{ "angle-template", 1, 0, 'A' },
			{ "dih-template", 1, 0, 'D' },
#endif
			{ NULL }
		};

		if ((c=getopt_long(argc, argv, "hvw:a:e:l:c:s:o:b:--", long_opts,
			&opt_ind)) == -1)
			break;
		switch(c)
		{
		case 'h':
		case '?':
			usage();
		case 'v':
			printf("Version: $Id: lmin_main.c,v 3.3 1998/03/21 00:35:53 kas Exp $\n");
			return 0;
		case 'a':
			accuracy = atof(optarg);
			break;
		case 'w':
			vdw14 = atof(optarg);
			break;
		case 'e':
			eel14 = atof(optarg);
			break;
		case 'l':
			diel_lin = atof(optarg);
			break;
		case 'c':
			diel_const = atof(optarg);
			break;
		case 'C':
			cutoff = atof(optarg);
			break;
		case 's':
			seed = atoi(optarg);
			break;
		case 'o':
			outfile = optarg;
			break;
		case 'b':
			infile = optarg;
			break;
#ifndef NONTRANSFORMED_ONLY
		case 'P':
			ptmpl = optarg;
			break;
		case 'A':
			atmpl = optarg;
			break;
		case 'D':
			ttmpl = optarg;
			break;
#endif
		default:
			usage();
			return -1;
		}
	}

	if (optind+1 > argc)
		usage();

	if ((ffp = prm2ff(argv[optind])) == NULL)
	{
		fprintf(stderr, "Cannot read force field from %s\n",
			argv[optind]);
		return 1;
	}

	setup_ff(ffp, vdw14, eel14, diel_lin, diel_const, cutoff);

	pvp = alloc_pv(ffp);

        if (infile)
                read_crd_file(pvp, infile);
        else {
		perror("Coordinate file required for local minimization.");
		exit(1);
	}

	minimize_l(pvp, accuracy); 

	printf("Minimum is %f\n", pvp->value);

	return 0;
}

static void usage()
{
	fprintf(stderr, "Usage: %s [options] file.top\n"
		"Where [options] is zero or more from\n"
		"--help\tprints out this message\n"
		"--version\tprints out version of program\n"
		"--vdw14 number\tsets the scaling parameter for VdW 1-4 interactions\n"
		"--eel14 number\tsets the scaling parameter for E-static 1-4 interactions\n"
		"--diel-lin number\tsets the linear part of dielectric funtion\n"
		"--diel-const number\tsets the constant part of dielectric function\n"
		"--cutoff number\tsets the 1991 force field cutoff\n"
		"--seed number\tseed for the random number generator\n"
		"--outfile filename.crd\twhere coordinates of minimum state will be written\n"
		"--begin-coords filename.crd\tstarting state of the molecule\n"
		"--accuracy number\taccuracy for the local minimizing procedure\n"
#ifndef NONTRANSFORMED_ONLY
		"--pair-template string\ttemplate for pair-interactions file\n"
		"--angle-template string\ttemplate for angle-interactions file\n"
		"--dih-template string\ttemplate for torsion-interactions file\n"
#endif
		,program_name);
	exit(1);
}

